@@ -12,6 +12,7 @@ from .config.project import ProjectConfig
 class Device():
     def __init__(self, project: ProjectConfig):
         self.config = project.global_config.device
+        self.framework_base = project.get_framework_base()
         self.container_mode = project.container_mode
         self.collect_configs(project)
 
@@ -20,6 +21,8 @@ class Device():
 
         if project.ssh:
             self.config.ipv4 = project.ssh
+            # in case project sets ssh without port, if project sets ssh, always copy port over
+            self.config.ssh_port = project.ssh_port
 
         if project.device_serial_number:
             self.config.serial_number = project.device_serial_number
@@ -68,16 +71,25 @@ class Device():
         if self.config.ipv4:
             dir_path = os.path.dirname(dst)
             self.run_command(f'mkdir -p {dir_path}')
-            command = f'scp {src} phablet@{self.config.ipv4}:{dst}'
+            command = ['scp']
+
+            # scp options has to be before src and dest
+            if self.config.ssh_port:
+                command += ['-o', f'Port={self.config.ssh_port}']
+
+            command += [f'{src}', f'phablet@{self.config.ipv4}:{dst}']
         else:
             adb_args = self.get_adb_args()
             command = f'adb {adb_args} push {src} {dst}'
 
-        run_subprocess_check_call(command, shell=True)
+        run_subprocess_check_call(command)
 
     def get_ssh_command(self, command, forward_port=None):
         ssh_args = ""
 
+        if self.config.ssh_port:
+            ssh_args = f"{ssh_args} -o Port={self.config.ssh_port}"
+
         if forward_port:
             ssh_args = f"{ssh_args} -L {forward_port}:localhost:{forward_port}"
 
@@ -93,9 +105,16 @@ class Device():
             self.forward_port_adb(forward_port, adb_args)
 
         if isinstance(command, list):
-            command = ";".join(command)
+            command = " && ".join(command)
 
-        return f'adb {adb_args} shell "{command}"'
+        if self.framework_base == '16.04':
+            logger.debug("Using UT 16.04 adb command")
+            adb_command = f'adb {adb_args} shell "{command}"'
+        else:
+            logger.debug("Using UT 20.04 adb command")
+            adb_command = f'echo "{command} || echo ADB_COMMAND_FAILED" | adb {adb_args} shell'
+
+        return adb_command
 
     def run_command(self, command, cwd=None, get_output=False, forward_port=None):
         if self.container_mode:
@@ -115,8 +134,11 @@ class Device():
 
         logger.debug("Running device command: %s", wrapped_command)
 
-        if get_output:
-            return run_subprocess_check_output(wrapped_command, cwd=cwd, shell=True)
+        output = run_subprocess_check_output(wrapped_command, cwd=cwd, shell=True)
 
-        run_subprocess_check_call(wrapped_command, cwd=cwd, shell=True)
+        if output.strip().endswith("ADB_COMMAND_FAILED"):
+            raise ClickableException("Command ran on device via ADB failed. See output above.")
+
+        if get_output:
+            return output
         return None
